# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv
from osv import fields
from tools import config

#----------------------------------------------------------
# Migrate
#----------------------------------------------------------
class migrate_migrate(osv.osv):
    _name = "migrate.migrate"
    _description = "Data Base Migrate"

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'module_ids': fields.one2many('migrate.migrate.module', 'migrate_id', 'Modules',),
        'state': fields.selection([('draft','Draft'), ('done','Done')], 'Status', readonly=True),
        'server': fields.char('Server', size=50),
        'port': fields.char('Port', size=5),
        'db': fields.char('Data Base', size=50),
        'login': fields.char('User Name', size=50),
        'password': fields.char('Password', size=54, password=True),
    }

    _defaults = {
        'state': lambda *a: 'draft',
        'server': lambda *a: 'localhost',
        'port': lambda *a: '8069',
    }

    def cancel(self,cr, uid, ids, context={}):
        return self.write(cr, uid, ids, {'state': 'draft'})

migrate_migrate()

#----------------------------------------------------------
# Migrate Modules
#----------------------------------------------------------
class migrate_migrate_module(osv.osv):
    _name = "migrate.migrate.module"
    _description = "Data Base Migrate Module"

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'migrate_id': fields.many2one('migrate.migrate', 'Migrate', required=True, select=True, ondelete='cascade'),
        'reg': fields.integer('Registers'),
        'include': fields.boolean('Include'),
        'replace': fields.boolean('Replace'),
        'seq': fields.integer('Sequence', required=True),
        'id_seq': fields.integer('IDSeq'),
        'model_org': fields.char('Model ORG', size=64),
        'model_dest': fields.char('Model DEST', size=64), #fields.many2one('ir.model', 'Model DEST'),
        'field_ids': fields.one2many('migrate.migrate.field', 'module_id', 'Fields'),
        'state': fields.selection([('draft','Draft'), ('field','Fields Loaded'), ('pending','Pending'), ('done','Done')], 'Status', readonly=True),
        'note': fields.text('Note'),
        'code': fields.boolean('Code'),
        'python': fields.text('Paython Code'),
        'localdic': fields.text('Localdic'),
    }

    _defaults = {
        'state': lambda *a: 'draft',
        'localdic': lambda *a: "{'cr_from': cr_from, 'module': module, 'pool': pooler.get_pool(cr.dbname), 'cr': cr, 'uid': uid}",
        'python': lambda *a: '''# result = "python code or constant" ''',
    }
    _order = 'seq,model_org'
#    _order = 'include desc,seq,model_org'

    def cancel(self,cr, uid, ids, context={}):
        return self.write(cr, uid, ids, {'state': 'draft'})

    def field(self,cr, uid, ids, context={}):
        return self.write(cr, uid, ids, {'state': 'field'})

migrate_migrate_module()

#----------------------------------------------------------
# Migrate Field
#----------------------------------------------------------
class migrate_migrate_field(osv.osv):
    _name = "migrate.migrate.field"
    _description = "Data Base Migrate Field"

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'module_id': fields.many2one('migrate.migrate.module', 'Migrate', required=True, select=True, ondelete='cascade'),
        'seq': fields.integer('Seq', required=True),
        'field_org': fields.char('Field ORG', size=64),
        'field_dest': fields.char('Field DEST', size=64), #fields.many2one('ir.model.fields', 'Field DEST'),
        'field_org_type': fields.char('Field ORG Type', size=64),
        'field_dest_type': fields.char('Field DEST Type', size=64),
        'include': fields.boolean('Include'),
        'order': fields.boolean('Order by'),
        'code': fields.boolean('Code'),
        'python': fields.text('Paython Code'),
        'localdic': fields.text('Localdic'),
        'include_reg': fields.text('Include Register'),
        'exclude_reg': fields.text('Exclude Register'),
        'pending': fields.boolean('Pending'),
    }

    _defaults = {
        'include': lambda *a: False,
        'localdic': lambda *a: "{'cr_from': cr_from, 'field': field, 'reg': reg, 'pool': pooler.get_pool(cr.dbname), 'cr': cr, 'uid': uid}",
        'python': lambda *a: '''# result = "python code or constant" ''',
    }

    _order = 'seq'

    def onchange_field_id(self, cr, uid, ids, field_dest):
        if not field_dest:
            result = {'value': {'field_dest_type': False}}
        else:
            ttype = self.pool.get('ir.model.fields').browse(cr, uid, field_dest).ttype
            result = {'value': {'field_dest_type': ttype}}
        return result

migrate_migrate_field()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

