# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import netsvc
import pooler
import wizard
import xmlrpclib
from osv import osv
import psycopg2
import psycopg2.extras
from tools import config


FORM = '''<?xml version="1.0"?>
<form string="Search Global Bank Accounts">
    <separator colspan="4"/>
    <field name="url" readonly="1"/>
    <newline/>
    <field name="db" />
    <newline/>
    <field name="login" required="1"/>
    <newline/>
    <field name="password" required="1"/>
        <separator string="Modules" colspan="4"/>
	        <field name="module_ids" width="450" height="150" colspan="2" nolabel="1" readonly="1"/>
</form>'''

FIELDS = {
    'url': {'string': 'url', 'type':'char', 'size': 64},
    'server': {'string': 'Server', 'type':'char', 'size': 64, 'required':True, 'default': lambda *a: 'localhost'},
    'port': {'string': 'Port', 'type':'char', 'size': 5, 'required':True, 'default': lambda *a: '8069'},
    'login': {'string': 'User Name', 'type':'char', 'size': 64, 'default': lambda *a: ''},
    'password': {'string': 'password', 'type':'char', 'password': True, 'size': 64, 'default': lambda *a: ''},
	'module_ids': {'string': 'Modules', 'type': 'one2many', 'relation': 'migrate.migrate.module'},
}

class load_field(wizard.interface):

    def _load_db(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        mod_obj = pool.get('migrate.migrate.module')
        field_obj = pool.get('migrate.migrate.field')
        field_dest_obj = pool.get('ir.model.fields')
        form = data['form']
        dbname, user, pwd, seq = form['db'], form['login'], form['password'], 0
        try:
            conn = psycopg2.connect("dbname="+dbname+" user="+user+" host="+form['server']+" password="+pwd+" ");
        except:
            raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the database !'))
        cr_from = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)#DictCursor)
        for m in form['module_ids']:
            field_obj.unlink(cr, uid, field_obj.search(cr, uid, [('module_id','=',m[1])]))
            module = mod_obj.browse(cr, uid, m[1])
            cr_from.execute("SELECT c.oid, n.nspname, c.relname FROM pg_catalog.pg_class c " \
                            "LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace " \
                            "WHERE c.relname ~ '^("+module.model_org+")$' AND pg_catalog.pg_table_is_visible(c.oid) ORDER BY 2, 3; ")
            res = cr_from.fetchone()
            cr_from.execute("SELECT a.attname, pg_catalog.format_type(a.atttypid, a.atttypmod), (SELECT substring(pg_catalog.pg_get_expr(d.adbin, " \
                            "d.adrelid) for 128) FROM pg_catalog.pg_attrdef d WHERE d.adrelid = a.attrelid AND d.adnum = a.attnum AND a.atthasdef), " \
                            "a.attnotnull, a.attnum FROM pg_catalog.pg_attribute a " \
                            "WHERE a.attrelid = "+str(res['oid'])+" AND a.attnum > 0 AND NOT a.attisdropped ORDER BY a.attnum")
            res = cr_from.fetchall()
            if module.model_dest:
                cr.execute("SELECT c.oid, n.nspname, c.relname FROM pg_catalog.pg_class c " \
                            "LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace " \
                            "WHERE c.relname ~ '^("+module.model_dest+")$' AND pg_catalog.pg_table_is_visible(c.oid) ORDER BY 2, 3; ")
                res_to = cr.dictfetchone()
                cr.execute("SELECT a.attname,pg_catalog.format_type(a.atttypid,a.atttypmod),(SELECT substring(pg_catalog.pg_get_expr(d.adbin, " \
                            "d.adrelid) for 128) FROM pg_catalog.pg_attrdef d WHERE d.adrelid = a.attrelid AND d.adnum = a.attnum AND a.atthasdef), " \
                            "a.attnotnull, a.attnum FROM pg_catalog.pg_attribute a " \
                            "WHERE a.attrelid = "+str(res_to['oid'])+" AND a.attnum > 0 AND NOT a.attisdropped ORDER BY a.attnum")
                res_to = cr.dictfetchall()
            else:
                raise wizard.except_wizard(_('Error'), _('There are not model destination !'))
            old_field = []
            for field in res:
                dest, dest_type = False, False 
                for field_to in res_to:
                    if field['attname'] == field_to['attname']:
                        dest = field_to['attname']
                        dest_type = field_to['format_type']
                old_field.append(field['attname'])
                field_obj.create(cr, uid, {
                    'name': field['attname'],
                    'module_id': m[1],
                    'seq': field['attnum'],
                    'field_org': field['attname'],
                    'field_dest': dest,
                    'field_org_type': field['format_type'],
                    'field_dest_type': dest_type,
                    'include': field['format_type'] == dest_type,
                })
            for field_to in res_to:
                if field_to['attname'] not in old_field:
                    field_obj.create(cr, uid, {
                    'name': field_to['attname'],
                    'module_id': m[1],
                    'seq': field_to['attnum'],
                    'field_org': False,
                    'field_dest': field_to['attname'],
                    'field_org_type': False,
                    'field_dest_type': field_to['format_type'],
                    })
                    
            mod_obj.write(cr, uid, m[1], {'state': 'field'})
        return {}

    def _load_db_1(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        mod_obj = pool.get('migrate.migrate.module')
        field_obj = pool.get('migrate.migrate.field')
        field_dest_obj = pool.get('ir.model.fields')
        form = data['form']
        dbname, user, pwd = form['db'], form['login'], form['password']
        uid_from = xmlrpclib.ServerProxy('http://%s:%s/xmlrpc/common'%(form['server'],form['port'])).login(dbname, user, pwd)
        rpc = xmlrpclib.ServerProxy('http://%s:%s/xmlrpc/object'%(form['server'],form['port']))
        ids, seq = [], 0
        for m in form['module_ids']:
            module = mod_obj.browse(cr, uid, m[1])
            ids = rpc.execute(dbname, uid_from, pwd, 'ir.model.fields', 'search', [('model_id','=',module.model_org_id)])
            field_obj.unlink(cr, uid, field_obj.search(cr, uid, [('module_id','=',m[1])]))
            for field in rpc.execute(dbname, uid_from, pwd, 'ir.model.fields', 'read', ids, ['name', 'ttype']):
                seq += 1
                dest = field_dest_obj.search(cr, uid, [('model_id','=',module.model_dest.id),('name','=',field['name'])])
                field_obj.create(cr, uid, {
                    'name': field['name'],
                    'module_id': m[1],
                    'seq': seq,
                    'field_org_id': field['id'],
                    'field_org': field['name'],
                    'field_dest': dest and dest[0],
                    'field_org_type': field['ttype'],
                    'field_dest_type': dest and field_dest_obj.browse(cr, uid, dest[0]).ttype or False,
                })
            mod_obj.write(cr, uid, m[1], {'state': 'field'})
        return {}

    def _get_connection_data(self, cr, uid, data, context):
        migrate = pooler.get_pool(cr.dbname).get('migrate.migrate').browse(cr, uid, data['id'])
        form = data['form']
        form['server'] = migrate.server
        form['port'] = migrate.port
        form['login'] = migrate.login
        form['password'] = migrate.password
        form['url'] = 'http://'+form['server']+':'+form['port']
        if 'db' not in FIELDS:
            try: 
                sock = xmlrpclib.ServerProxy(form['url']+'/xmlrpc/db')
                selection = [(x, x) for x in getattr(sock, 'list')(*())]
                FIELDS['db'] = {'string': 'Database', 
                              'type': 'selection',
                              'selection': selection,
                              'required': True}
            except:
                raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the old database !'))
        if migrate.db:
            form['db'] = migrate.db
        module_ids = []
        for module in migrate.module_ids:
            if module.include and module.state == 'draft':
                module_ids.append(module.id)
        form['module_ids'] = module_ids
        return form

    states = {
		'init': {
			'actions': [_get_connection_data],
			'result': {
				'type': 'form',
				'arch': FORM,
				'fields': FIELDS,
				'state': [('end', 'Close','gtk-cancel'),('db','Load Fields','gtk-go-forward')]
			}
		},
        'db': {
            'actions': [],
            'result': {'type': 'action', 'action':_load_db, 'state':'end'}
        },

    }

load_field('migrate.load.field')


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

