# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import netsvc
import pooler
import wizard
import xmlrpclib
from osv import osv
import psycopg2
import psycopg2.extras
from tools import config

FIELDS = {}

FORM = '''<?xml version="1.0"?>
<form string="Search Global Bank Accounts">
    <separator colspan="4"/>
    <field name="server" />
    <newline/>
    <field name="port" />
</form>'''

CONNECT_FORM = '''<?xml version="1.0"?>
<form string="Search Global Bank Accounts">
    <separator colspan="4"/>
    <field name="url" readonly="1"/>
    <newline/>
    <field name="db" />
    <newline/>
    <field name="login" required="1"/>
    <newline/>
    <field name="password" required="1"/>
</form>'''

db_form = '''<?xml version="1.0"?>
<form string="Select DB Connection Information">
    <separator colspan="4"/>
    <field name="server"/>
    <newline/>
    <field name="port"/>
    <newline/>
    <field name="db"/>
</form>'''

db_fields = {
    'server': {'string': 'Server', 'type': 'char', 'required': True},
    'port': {'string': 'port', 'type': 'char', 'size': 5, 'required': True},
    'db': {'string': 'Data Base', 'type': 'char', 'required': True, 'default': lambda *a: 'bias'},
}

class load_module(wizard.interface):

    def _load_db(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        form = data['form']
        dbname, user, pwd = form['db'], form['login'], form['password']
        ids = pool.get('migrate.migrate.module').search(cr, uid, [('migrate_id','=',data['id'])])
        pool.get('migrate.migrate.module').unlink(cr, uid, ids)
        try:
            conn = psycopg2.connect("dbname="+dbname+" user="+user+" host="+form['server']+" password="+pwd+" ");
        except:
            raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the database !'))
        cr_from = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        cr_from.execute("SELECT tablename from pg_tables where schemaname = 'public' order by tablename")
        cr.execute("SELECT tablename from pg_tables where schemaname = 'public' order by tablename")
        tables = [x[0] for x in cr.fetchall()]
        for row in cr_from.fetchall():
            cr_from.execute("SELECT * from "+row['tablename']+" ")
            if cr_from.rowcount:
                res = cr_from.fetchall()
                pool.get('migrate.migrate.module').create(cr, uid, {
                    'name': row['tablename'],
                    'migrate_id': data['id'],
                    'reg': len(res),
                    'include': False,
                    'seq': row['tablename'].startswith('ir') and 30 or row['tablename'].startswith('res') and 10 or 20,
                    'model_org': row['tablename'],
                    'model_dest': row['tablename'] in tables and row['tablename'] or False,
                }) 
        pool.get('migrate.migrate').write(cr, uid, data['id'], {
                    'state': 'done',
                    'server': form['server'],
                    'port': form['port'],
                    'db': form['db'],
                    'login': form['login'],
                    'password': form['password'],
                    })
        return {}


    def _load_db_1(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        form = data['form']
        dbname = form['db']
        user = form['login']
        pwd = form['password']
        rpc = xmlrpclib.ServerProxy('http://%s:%s/xmlrpc/common'%(form['server'],form['port']))
        uid_from = rpc.login(dbname, user, pwd)
        rpc = xmlrpclib.ServerProxy('http://%s:%s/xmlrpc/object'%(form['server'],form['port']))
        ids = pool.get('migrate.migrate.module').search(cr, uid, [('migrate_id','=',data['id'])])
        pool.get('migrate.migrate.module').unlink(cr, uid, ids)
        ids = rpc.execute(dbname, uid_from, pwd, 'ir.model', 'search', [])
        for module in rpc.execute(dbname, uid_from, pwd, 'ir.model', 'read', ids, ['model','name']):
            reg = rpc.execute(dbname, uid_from, pwd, module['model'], 'search', [])
            if reg:
                field = rpc.execute(dbname, uid_from, pwd, 'ir.model.fields', 'search', [('model_id','=',module['id'])])
                module_5 = pool.get('ir.model').search(cr, uid, [('model','=',module['model'])])
                args = [('model_id','=',module['id']),('ttype','in', ['many2one', 'one2many'])]
                rel_one = rpc.execute(dbname, uid_from, pwd, 'ir.model.fields', 'search', args)
                args = [('model_id','=',module['id']),('ttype','in', ['many2many'])]
                rel_m2m = rpc.execute(dbname, uid_from, pwd, 'ir.model.fields', 'search', args)
                seq = \
                    module['model'].startswith('ir') and 600 or \
                    (not len(rel_one) and not len(rel_m2m)) and 200 or \
                    module['model'].startswith('res') and 300 or \
                    (len(rel_one) and not len(rel_m2m)) and 400 or \
                    (not len(rel_one) and len(rel_m2m)) and 500 \
                    or 50
                pool.get('migrate.migrate.module').create(cr, uid, {
                    'name': module['name'],
                    'migrate_id': data['id'],
                    'reg': len(reg),
                    'include': False,
                    'seq': seq,
                    'model_org': module['model'],
                    'model_org_id': module['id'],
                    'model_dest': module_5 and module_5[0],
                }) 
        pool.get('migrate.migrate').write(cr, uid, data['id'], {
                    'state': 'done',
                    'server': form['server'],
                    'port': form['port'],
                    'db': form['db'],
                    'login': form['login'],
                    'password': form['password'],
                    })
        return {}

    def _get_fields(self, cr, uid, data, context):
        migrate = pooler.get_pool(cr.dbname).get('migrate.migrate').browse(cr, uid, data['id'])
        form = data['form']
        while FIELDS:
            FIELDS.popitem()
        FIELDS['url'] = {'string': 'url', 'type':'char', 'size': 64}
        FIELDS['server'] = {'string': 'Server', 'type':'char', 'size': 64, 'required':True, 'default': lambda *a: 'localhost'}
        FIELDS['port'] = {'string': 'Port', 'type':'char', 'size': 5, 'required':True, 'default': lambda *a: '8069'}
        FIELDS['login'] = {'string': 'User Name', 'type':'char', 'size': 64, 'default': lambda *a: ''}
        FIELDS['password'] = {'string': 'password', 'type':'char', 'password': True, 'size': 64, 'default': lambda *a: ''}
        form['server'] = migrate.server
        form['port'] = migrate.port
        form['login'] = migrate.login
        form['password'] = migrate.password
        return form

    def _list_db(self, cr, uid, data, context={}):
        migrate = pooler.get_pool(cr.dbname).get('migrate.migrate').browse(cr, uid, data['id'])
        form = data['form']
        form['url'] = 'http://'+form['server']+':'+form['port']
        if 'db' not in FIELDS:
            try: 
                sock = xmlrpclib.ServerProxy(form['url']+'/xmlrpc/db')
                selection = [(x, x) for x in getattr(sock, 'list')(*())]
                FIELDS['db'] = {'string': 'Database', 
                              'type': 'selection',
                              'selection': selection,
                              'required': True}
            except:
                raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the old database !'))
        if migrate.db:
            form['db'] = migrate.db
        return form

    states = {
		'init': {
			'actions': [_get_fields],
			'result': {
				'type': 'form',
				'arch': FORM,
				'fields': FIELDS,
				'state': [('end', 'Close','gtk-cancel'),('connect','Connect','gtk-go-forward')]
			}
		},
        'connect': {
            'actions': [_list_db],
			'result': {
				'type': 'form',
				'arch': CONNECT_FORM,
				'fields': FIELDS,
				'state': [('end', 'Close','gtk-cancel'),('db','Next','gtk-go-forward')]
			}
		},
        'db': {
            'actions': [],
            'result': {'type': 'action', 'action':_load_db, 'state':'end'}
        },

    }

load_module('migrate.load.module')


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

