# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import netsvc
import pooler
import wizard
import xmlrpclib
from osv import osv
import psycopg2
from tools import config
from string import lower
from re import sub

COUNT = {'i':0,'res':0,'init':0,'table_id':0,'end':0}

FORM = '''<?xml version="1.0"?>
<form string="Migrate">
    <separator colspan="4"/>
    <field name="url" readonly="1"/>
    <newline/>
    <field name="db" />
    <newline/>
    <field name="login" required="1"/>
    <newline/>
    <field name="password" required="1"/>
    <newline/>
    <field name="count"/>
    <newline/>
    <field name="initial_id"/>
    <separator string="Modules" colspan="4"/>
        <field name="module_ids" width="450" height="150" colspan="2" nolabel="1" readonly="1"/>
</form>'''

FIELDS = {
    'url': {'string': 'url', 'type':'char', 'size': 64},
    'server': {'string': 'Server', 'type':'char', 'size': 64, 'required':True, 'default': lambda *a: 'localhost'},
    'port': {'string': 'Port', 'type':'char', 'size': 5, 'required':True, 'default': lambda *a: '8069'},
    'login': {'string': 'User Name', 'type':'char', 'size': 64, 'default': lambda *a: ''},
    'password': {'string': 'password', 'type':'char', 'password': True, 'size': 64, 'default': lambda *a: ''},
	'module_ids': {'string': 'Modules', 'type': 'one2many', 'relation': 'migrate.migrate.module'},
    'count': {'string': 'Count Control', 'type':'integer', 'default': lambda *a: 1000},
    'initial_id': {'string': 'Initial ID', 'type':'integer'},    
}

FORM_REPLACE = '''<?xml version="1.0"?>
<form string="Select Type of Replace">
    <separator colspan="4"/>
    <field name="replace"/>
</form>'''

FIELDS_REPLACE = {
    'replace':{
        'string':"Select Replace Type",
        'type':'selection',
        'selection':[('replace','Replace Only'),('add','Replace & Add if not present')],
        'required':False,
        'default': lambda *a: 'add',
    },
}

FORM_END = '''<?xml version="1.0"?>
<form string="Migration Complited">
    <separator colspan="4"/>
    <field name="count"/>
</form>'''

FIELDS_END = {
    'count': {'string': 'Register Processed', 'type':'integer'},
}

class migrate_module(wizard.interface):

    def _load_db(self, cr, uid, data, context):
        mod_obj = pooler.get_pool(cr.dbname).get('migrate.migrate.module')
        cr_from = self._get_cr_from(cr, uid, data, context)
        for m in data['form']['module_ids']:
            module = mod_obj.browse(cr, uid, m[1])
            if m[2]['include']:
                limit, query_fields, order, other_fields, include_reg, exclude_reg, pending = "", "", "", [], [], [], False
                if module.code:
                    COUNT['res'] = 1
                    res = []

                    localdic = eval(module.localdic)
                    exec module.python in localdic
                    result = localdic.get('result', False)
                    result = result and [x[0] for x in result]
                    print 'execute', result

                    return data['form']
                for field in module.field_ids:
                    if field.include and field.field_org:
                        if field.order:
                            order = "order by "+field.field_org
                        query_fields += query_fields and (", "+(field.field_org)) or field.field_org
                        if field.name == 'id':
                            COUNT['table_id'] = 1
                    elif field.include and not field.field_org:
                        other_fields += [field]
                    if field.include_reg and not include_reg:
                        include_reg = field
                    if field.exclude_reg and not exclude_reg:
                        exclude_reg = field
                    if field.pending:
                        pending = True
                if COUNT['table_id']:
                    limit = " where id > "+str(COUNT['i'])+" and id <= "+str(COUNT['i']+data['form']['count'])+" "
                if COUNT['end']:
                    res = []
                else:
                    cr_from.execute("select "+query_fields+" from "+m[2]['model_org']+limit+order)
                    res = cr_from.fetchall()
                if not res:
                    COUNT['res'] = 1
                    print 'NO RES'
                    if pending:
                        mod_obj.write(cr, uid, m[1], {'state': 'pending'})
                    else:
                        mod_obj.write(cr, uid, m[1], {'state': 'done'})
                    return data['form']
                query_fields = self._get_query_fields(cr, uid, res, other_fields)
                for reg in res:
                    COUNT['i'] = COUNT['i'] + 1
                    if self._check_include(cr, uid, reg, include_reg, exclude_reg):
                        continue
                    elif module.replace:
                        if not self._write_reg(cr, uid, data, reg, m, other_fields, cr_from):
                            self._insert_reg(cr, uid, reg, m, other_fields, cr_from, query_fields)
                    else:                       
                        self._insert_reg(cr, uid, reg, m, other_fields, cr_from, query_fields)
        return data['form']

    def _write_reg(self, cr, uid, data, reg, m, other_fields, cr_from, context={}):
        module = pooler.get_pool(cr.dbname).get('migrate.migrate.module').browse(cr, uid, m[1])
        field = filter(lambda x: x['name']=='id', module.field_ids)
        if not field:
            raise wizard.except_wizard(_('Table Error'), _('This table do not contain id field !'))
        query, reg_id = "", False
        for key in reg.keys():
            if key == 'id':
                reg_id = reg[key]
            query = self._get_query(cr, uid, module, key, reg, query, cr_from)
        for key in other_fields:
            if key.code and key.python:
                query = self._get_query(cr, uid, module, key.name, reg, query, cr_from)
        query = "UPDATE "+m[2]['model_dest']+" SET "+query+" WHERE id = "+str(reg_id)+" "
#        print 'query=',query
        cr.execute("SELECT id FROM "+m[2]['model_dest']+" WHERE id = "+str(reg_id)+" ")
        if not cr.fetchone() and data['form']['replace'] == 'add':
            return False
        cr.execute(query)
        return True

    def _insert_reg(self, cr, uid, reg, m, other_fields, cr_from, query_fields, context={}):
        module = pooler.get_pool(cr.dbname).get('migrate.migrate.module').browse(cr, uid, m[1])
        values, s, reg_id = [], "", False
        for key in reg.keys():
            values, s = self._get_data_values(cr, uid, module, key, reg, s, values, cr_from)
        for key in other_fields:
            if key.code and key.python:
                values, s = self._get_data_values(cr, uid, module, key.name, reg, s, values, cr_from)
        query = "INSERT INTO "+m[2]['model_dest']+" ("+query_fields+") values ("+s+") "
#        print (query%tuple(values))
        cr.execute(query%tuple(values))
        return True

    def _get_query(self, cr, uid, module, key, reg, query, cr_from, context={}):
        field = filter(lambda x: key == x['name'], module.field_ids)
        t = field[0].field_dest_type
        if (key in reg) and (reg[key] == None) and not field[0].code:
            query += query and ", "+key+" = "+'Null' or key+" = "+'Null'
        elif t.startswith('character') or t == 'path' or t.startswith('text') or t.startswith('time') or t.startswith('date'):
            val = self._applicable(cr, uid, field[0], reg, cr_from, context)
            query = self._parser_val(cr, uid, val, reg, key, query, text=True)
        else:
            val = self._applicable(cr, uid, field[0], reg, cr_from, context)
            query = self._parser_val(cr, uid, val, reg, key, query, text=False)
        return query

    def _parser_val(self, cr, uid, val, reg, key, query, text):
        res = False
        if text and val and lower(val) in ('null','false'):
            res = False
        elif text and (val and lower(val) =='zero'):
            res = 0
        elif (type(val).__name__ == 'int' and val == 0) or (type(val).__name__ == 'float' and val == 0):
            res = 0
        elif val:
            res = val
        elif key in reg:
            res = reg[key]
        if text:
            if query:
                query += ", "+key+" = '%s'"%(sub("'",'',res))
            else:
                query += key+" = '%s'"%(sub("'",'',res))
        else:
            if query:
                query += ", "+key+" = %s"%(res)
            else:
                query += key+" = %s"%(res)
        return query

    def _get_data_values(self, cr, uid, module, key, reg, s, values, cr_from, context={}):
        field = filter(lambda x: key == x['name'], module.field_ids)
        t = field[0].field_dest_type
        if (key in reg) and (reg[key] == None) and not field[0].code:
            s += s and ",%s" or "%s"
            values.append('Null')
        elif t.startswith('character') or t == 'path' or t.startswith('text') or t.startswith('time') or t.startswith('date'):
            s += s and ",'%s'" or "'%s'"
            val = self._applicable(cr, uid, field[0], reg, cr_from, context)
            values.append(self._parser_data_val(cr, uid, val, reg, key, text=True)) #val=='Null' and False or val or reg[key])
        else:
            s += s and ",%s" or "%s"
            val = self._applicable(cr, uid, field[0], reg, cr_from, context)
            values.append(self._parser_data_val(cr, uid, val, reg, key, text=False)) #val=='Null' and False or val or reg[key])
        return values, s

    def _parser_data_val(self, cr, uid, val, reg, key, text):
        res = False
        if text and val and lower(val) in ('null','false'):
            res = False
        elif text and (val and lower(val) =='zero') or (type(val).__name__ == 'int' and val == 0):
            res = 0
        elif val:
            res = val
        if text:
            if key in reg:
                res = sub("'",'',reg[key])
        else:
            if key in reg:
                res = reg[key]
        return res

    def _applicable(self, cr, uid, field, reg, cr_from, context):
        localdic = {}
        if field.code:
            localdic = eval(field.localdic)
            exec field.python in localdic
        return localdic.get('result', False)

    def _get_cr_from(self, cr, uid, data, context):
        form = data['form']
        dbname, user, pwd = form['db'], form['login'], form['password']
        try:
            conn = psycopg2.connect("dbname="+dbname+" user="+user+" host="+form['server']+" password="+pwd+" ");
        except:
            raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the database !'))
        cr_from = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)#DictCursor)
        return cr_from

    def _get_query_fields(self, cr, uid, res, other_fields, context={}):
        query_fields = ""
        for key in res[0].keys():
            query_fields += query_fields and ', '+key or key
        for key in other_fields:
            query_fields += query_fields and ', '+key.field_dest
        return query_fields

    def _check_include(self, cr, uid, reg, include_reg, exclude_reg, context={}):
        if exclude_reg:
            if reg[exclude_reg.field_org] in eval(exclude_reg.exclude_reg):
                return True
        elif include_reg:
            pass
        return False

    def _check(self, cr, uid, data, context):
        if COUNT['init'] == 0:
            COUNT['init'] = 1
            COUNT['i'] = data['form']['initial_id']
        mod_obj = pooler.get_pool(cr.dbname).get('migrate.migrate.module')
        try:
            for m in data['form']['module_ids']:
                module = mod_obj.browse(cr, uid, m[1])
                if m[2]['include'] and m[2]['replace']:
                    return 'replace'
                else:
                    return 'db'
        except:
            raise wizard.except_wizard(_('Module Error'), _('No module selected !'))
            raise wizard.except_wizard(_('Error !'), _("Hay líneas de asiento ya conciliadas!"))
    def _check_end(self, cr, uid, data, context):
        print COUNT
        if not COUNT['table_id']:
            COUNT['end'] = 1
        if COUNT['res'] == 1:
            return 'migration_end'
        return 'db'

    def _load_counters(self, cr, uid, data, context):
        form = data['form']
        form['count'] = COUNT['i']
        COUNT['i'] = 0
        COUNT['res'] = 0
        COUNT['init'] = 0
        COUNT['table_id'] = 0
        COUNT['end'] = 0
        return form

    def _get_connection_data(self, cr, uid, data, context):
        migrate = pooler.get_pool(cr.dbname).get('migrate.migrate').browse(cr, uid, data['id'])
        form = data['form']
        form['server'] = migrate.server
        form['port'] = migrate.port
        form['login'] = migrate.login
        form['password'] = migrate.password
        form['url'] = 'http://'+form['server']+':'+form['port']
        if 'db' not in FIELDS:
            try: 
                sock = xmlrpclib.ServerProxy(form['url']+'/xmlrpc/db')
                selection = [(x, x) for x in getattr(sock, 'list')(*())]
                FIELDS['db'] = {'string': 'Database', 
                              'type': 'selection',
                              'selection': selection,
                              'required': True}
            except:
                raise wizard.except_wizard(_('Connection Error'), _('I am unable to connect to the database !'))
        if migrate.db:
            form['db'] = migrate.db
        module_ids = []
        for module in migrate.module_ids:
            if module.include and module.state in ['field','pending']:
                module_ids.append(module.id)
        form['module_ids'] = module_ids
        return form

    states = {
		'init': {
			'actions': [_get_connection_data],
			'result': {
				'type': 'form',
				'arch': FORM,
				'fields': FIELDS,
				'state': [('end', 'Close','gtk-cancel'),('check','Migrate Register','gtk-go-forward')]
			}
		},
        'check': {
            'actions': [],
            'result': {'type':'choice','next_state':_check}
        },
        'replace': {
            'actions': [],
			'result': {
				'type': 'form',
				'arch': FORM_REPLACE,
				'fields': FIELDS_REPLACE,
				'state': [('end', 'Close','gtk-cancel'),('db','Migrate Register','gtk-go-forward')]
			}
        },
        'check_end': {
            'actions': [],
            'result': {'type':'choice','next_state':_check_end}
        },
        'db': {
            'actions': [],
            'result': {'type': 'action', 'action':_load_db, 'state':'check_end'}
        },
        'migration_end': {
            'actions': [_load_counters],
			'result': {
				'type': 'form',
				'arch': FORM_END,
				'fields': FIELDS_END,
				'state': [('end', 'Close','gtk-cancel')]
			}
        },

    }

migrate_module('migrate.migrate.module')


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

